#!/usr/bin/env python
from ncurses.form import *
from string import letters, digits, strip

letters = letters + digits + ' '

def main():
  stdscr = initscr() # save stdscr to a variable or it will be GC'ed
  cbreak() # provide unbuffered input
  noecho() # turn off input echoing
  nonl() # turn off newline translation
  intrflush(FALSE) # turn off flush-on-interrupt
  #form_set_update_mode('curses') # we're not using panels

  # create windows
  keys_win = WINDOW(3, COLS(), LINES() - 4, 0)
  form_win = WINDOW(LINES() - 4, COLS(), 0, 0)
  form_win.keypad(TRUE) # turn on keypad
  keys_win.move(0,0)
  keys_win.hline(ACS_HLINE(), COLS())
  keys_win.move(2,5)
  keys_win.addstr('TAB - Next Field   ALT-TAB - Last Field   ENTER - Finish' +
                  ' form')
  keys_win.refresh()

  # create fields
  fields = []
  fields.append(LABEL(0, 0, 'Last Name:'))
  fields.append(LABEL(1, 0, 'Age:'))
  fields.append(LABEL(2, 0, 'Comments:'))
  fields.append(FIELD(1, 10, 0, 11))
  fields[-1].field_opts_off(O_STATIC)
#  fields[-1].set_field_pad(ACS_CKBOARD())
  fields[-1].TYPE_ALPHA(1) # character data, minimum length one
  fields.append(FIELD(1, 5, 1, 6))
  fields[-1].field_opts_off(O_STATIC)
#  fields[-1].set_field_pad(ACS_CKBOARD())
  fields[-1].TYPE_INTEGER(2, 18, 68) # must be a 2 digit number from 18 ro 68
  fields.append(FIELD(3, COLS() - 4, 4, 2))
  fields[-1].field_opts_off(O_STATIC)
#  fields[-1].set_field_pad(ACS_CKBOARD())

  form = FORM(fields)
  form.set_form_win(form_win)
  form.set_form_sub(form_win)
  form.post_form()

  # input loop
  while TRUE:
    key = form_win.getch()
    if key == KEY_LEFT:
      form.form_driver(REQ_PREV_CHAR)
    elif key == KEY_RIGHT:
      form.form_driver(REQ_NEXT_CHAR)
    elif key == KEY_DOWN:
      form.form_driver(REQ_DOWN_CHAR)
    elif key == KEY_UP:
      form.form_driver(REQ_UP_CHAR)
    elif key == KEY_BACKSPACE:
      form.form_driver(REQ_DEL_PREV)
    key = chtype2chr(key) # convert to string
    if key == '\t': # tab
      form.form_driver(REQ_NEXT_FIELD)
    elif key == '\015': # return key
      ## don't accept return key on an field that fails validation
      if form.form_driver(REQ_NEXT_FIELD) != E_INVALID_FIELD:
        break # leave loop on return
    elif key in letters:
      form.form_driver(ord(key))

  form.form_driver(REQ_NEXT_FIELD) # makes sure buffer is up-to-date
  form.unpost_form()
  del form

  endwin()
  print "Name: ", strip(fields[3].field_buffer())
  print "Age: ", strip(fields[4].field_buffer())
  print "Comments: \n", strip(fields[5].field_buffer())
  
main()
