#!/usr/bin/env python
from ncurses.menu import *

def main():
  stdscr = initscr() # save stdscr to a variable or it will be GC'ed
  cbreak() # provide unbuffered input
  noecho() # turn off input echoing
  nonl() # turn off newline translation
  intrflush(FALSE) # turn off flush-on-interrupt
  keypad(TRUE) # turn on keypad mode
  curs_set(0) # make cursor invisible this doesn't work on all terminals

  stdscr.line_border()
  stdscr.move(LINES() - 2, 1)
  stdscr.addstr('Press space to select, arrow and Home/End to navigate')

  items = [] # initialize item list
  for count in xrange(1, 4): # create items
    items.append(ITEM('Choice ' + str(count), 'This is choice ' + str(count)))
  items.append(ITEM('EXIT', 'Get outta here!'))
  menu = MENU(items)
  (min_rows, min_cols) = menu.scale_menu() # find how big the window must be
  menu_window = PANEL(min_rows + 2, min_cols + 2, 2, 2) # make border window
  menu_window.line_border()
  # make subwindow
  menu_subwindow = menu_window.derwin(min_rows, min_cols, 1, 1)
  menu.set_menu_win(menu_window)
  menu.set_menu_sub(menu_subwindow)
  menu.post_menu()

  message_window = PANEL(5, 40, LINES() - 7, COLS() - 45)
  message_window.line_border()
  message_window.move(2,2)
  message_window.addstr("No choice made.")

  doupdate()

  # input loop
  while TRUE:
    req = ''
    ch = getch()
    if ch == KEY_UP:
      req = REQ_UP_ITEM
    elif ch == KEY_DOWN:
      req = REQ_DOWN_ITEM
    elif ch == KEY_HOME: # HOME and END don't seem to work on many x-terms
      req = REQ_FIRST_ITEM # they work fine on a Linux VT
    elif ch == KEY_END:
      req = REQ_LAST_ITEM
    elif ch == ord(' '): # space bar
      choice = menu.current_item(items).name # get selection
      if choice == 'EXIT': # if exit was chosen exit
        menu.unpost_menu()
        del menu # prevents harmless but annoying error message at exit
        break
      message_window.clear()
      message_window.line_border()
      message_window.move(2,2)
      message_window.addstr(choice) # display choice
      doupdate()
    if req: # if a valid command was made
      menu.menu_driver(req)
      doupdate()
    
  
main()
